@@ -148,7 +148,7 @@ asprintf(char ** ret, const char *format, ...)
  * for a reply, and while calling IceProcessMessages, a callback can be
  * invoked which will wait for another reply).  We take advantage of the
  * fact that for a given protocol, we are guaranteed that messages are
- * processed in the order we sent them.  So, everytime we have a new
+ * processed in the order we sent them.  So, every time we have a new
  * replyWait, we add it to the END of the 'saved_reply_waits' list.  When
  * we read a message and want to see if it matches a replyWait, we use the
  * FIRST replyWait in the list with the major opcode of the message.  If the
@@ -160,7 +160,7 @@ asprintf(char ** ret, const char *format, ...)
  * The return value of IceProcessMessages is one of the following:
  *
  * IceProcessMessagesSuccess - the message was processed successfully.
- * IceProcessMessagesIOError - an IO error occured.  The caller should
+ * IceProcessMessagesIOError - an IO error occurred.  The caller should
  *			       invoked IceCloseConnection.
  * IceProcessMessagesConnectionClosed - the connection was closed as a
  *					result of shutdown negotiation.
@@ -207,7 +207,7 @@ IceProcessMessages (
     if (!iceConn->io_ok)
     {
 	/*
-	 * An unexpected IO error occured.  The caller of IceProcessMessages
+	 * An unexpected IO error occurred.  The caller of IceProcessMessages
 	 * should call IceCloseConnection which will cause the watch procedures
 	 * to be invoked and the ICE connection to be freed.
 	 */
@@ -446,12 +446,15 @@ AuthRequired (
 
     pMsg->authIndex = authIndex;
     pMsg->authDataLength = authDataLen;
-    pMsg->length += WORD64COUNT (authDataLen);
 
-    IceWriteData (iceConn, authDataLen, (char *) authData);
+    if (authDataLen) {
+	pMsg->length += WORD64COUNT (authDataLen);
 
-    if (PAD64 (authDataLen))
-	IceWritePad (iceConn, PAD64 (authDataLen));
+	IceWriteData (iceConn, authDataLen, (char *) authData);
+
+	if (PAD64 (authDataLen))
+	    IceWritePad (iceConn, PAD64 (authDataLen));
+    }
 
     IceFlush (iceConn);
 }
@@ -526,10 +529,15 @@ AcceptConnection (
 
     pMsg->versionIndex = versionIndex;
 
-    STORE_STRING (pData, IceVendorString);
-    STORE_STRING (pData, IceReleaseString);
+    if (pData != NULL) {
+	STORE_STRING (pData, IceVendorString);
+	STORE_STRING (pData, IceReleaseString);
 
-    IceFlush (iceConn);
+	IceFlush (iceConn);
+    } else {
+	SEND_STRING (iceConn, IceVendorString);
+	SEND_STRING (iceConn, IceReleaseString);
+    }
 
     iceConn->connection_status = IceConnectAccepted;
 }
@@ -559,10 +567,15 @@ AcceptProtocol (
     pMsg->protocolOpcode = myOpcode;
     pMsg->versionIndex = versionIndex;
 
-    STORE_STRING (pData, vendor);
-    STORE_STRING (pData, release);
+    if (pData != NULL) {
+	STORE_STRING (pData, vendor);
+	STORE_STRING (pData, release);
 
-    IceFlush (iceConn);
+	IceFlush (iceConn);
+    } else {
+	SEND_STRING (iceConn, vendor);
+	SEND_STRING (iceConn, release);
+    }
 
 
     /*
@@ -609,7 +622,7 @@ ProcessError (
     IceReadCompleteMessage (iceConn, SIZEOF (iceErrorMsg),
 	iceErrorMsg, message, pStart);
 
-    if (!IceValidIO (iceConn))
+    if (!IceValidIO (iceConn) || pStart == NULL)
     {
 	IceDisposeCompleteMessage (iceConn, pStart);
 	return (0);
@@ -890,7 +903,7 @@ ProcessConnectionSetup (
     IceReadCompleteMessage (iceConn, SIZEOF (iceConnectionSetupMsg),
 	iceConnectionSetupMsg, message, pStart);
 
-    if (!IceValidIO (iceConn))
+    if (!IceValidIO (iceConn) || pStart == NULL)
     {
 	IceDisposeCompleteMessage (iceConn, pStart);
 	return (0);
@@ -934,7 +947,15 @@ ProcessConnectionSetup (
     if (hisAuthCount > 0)
     {
 	hisAuthNames = malloc (hisAuthCount * sizeof (char *));
-	EXTRACT_LISTOF_STRING (pData, swap, hisAuthCount, hisAuthNames);
+	if (hisAuthNames != NULL)
+	{
+	    EXTRACT_LISTOF_STRING (pData, swap, hisAuthCount, hisAuthNames);
+	}
+	else
+	{
+	    SKIP_LISTOF_STRING (pData, swap, hisAuthCount, pEnd, break);
+	    hisAuthCount = 0;
+	}
     }
 
     hisVersionCount = message->versionCount;
@@ -1142,7 +1163,7 @@ ProcessAuthRequired (
     IceReadCompleteMessage (iceConn, SIZEOF (iceAuthRequiredMsg),
 	iceAuthRequiredMsg, message, authData);
 
-    if (!IceValidIO (iceConn))
+    if (!IceValidIO (iceConn) || authData == NULL)
     {
 	IceDisposeCompleteMessage (iceConn, authData);
 	return (0);
@@ -1320,7 +1341,7 @@ ProcessAuthReply (
     iceAuthReplyMsg 	*message;
     int			replyDataLen;
     IcePointer		replyData;
-    int 		authDataLen;
+    int 		authDataLen = 0;
     IcePointer 		authData = NULL;
     char		*errorString = NULL;
 
@@ -1331,7 +1352,7 @@ ProcessAuthReply (
     IceReadCompleteMessage (iceConn, SIZEOF (iceAuthReplyMsg),
 	iceAuthReplyMsg, message, replyData);
 
-    if (!IceValidIO (iceConn))
+    if (!IceValidIO (iceConn) || replyData == NULL)
     {
 	IceDisposeCompleteMessage (iceConn, replyData);
 	return (0);
@@ -1475,7 +1496,7 @@ ProcessAuthReply (
 	    _IceProcessMsgInfo *process_msg_info;
 	    IcePointer clientData = NULL;
 	    char *failureReason = NULL;
-	    Status status = 1;
+	    Status setupStatus = 1;
 
 	    protocolSetupProc = myProtocol->protocol_setup_proc;
 	    protocolActivateProc = myProtocol->protocol_activate_proc;
@@ -1486,7 +1507,7 @@ ProcessAuthReply (
 		 * Notify the client of the Protocol Setup.
 		 */
 
-		status = (*protocolSetupProc) (iceConn,
+		setupStatus = (*protocolSetupProc) (iceConn,
 		    myProtocol->version_recs[iceConn->protosetup_to_me->
 		        my_version_index].major_version,
 		    myProtocol->version_recs[iceConn->protosetup_to_me->
@@ -1505,7 +1526,7 @@ ProcessAuthReply (
 		iceConn->protosetup_to_me->his_release = NULL;
 	    }
 
-	    if (status != 0)
+	    if (setupStatus != 0)
 	    {
 		/*
 		 * Send the Protocol Reply
@@ -1625,7 +1646,7 @@ ProcessAuthNextPhase (
     IceReadCompleteMessage (iceConn, SIZEOF (iceAuthNextPhaseMsg),
 	iceAuthNextPhaseMsg, message, authData);
 
-    if (!IceValidIO (iceConn))
+    if (!IceValidIO (iceConn) || authData == NULL)
     {
 	IceDisposeCompleteMessage (iceConn, authData);
 	return (0);
@@ -1752,7 +1773,7 @@ ProcessConnectionReply (
     IceReadCompleteMessage (iceConn, SIZEOF (iceConnectionReplyMsg),
 	iceConnectionReplyMsg, message, pStart);
 
-    if (!IceValidIO (iceConn))
+    if (!IceValidIO (iceConn) || pStart == NULL)
     {
 	IceDisposeCompleteMessage (iceConn, pStart);
 	return (0);
@@ -1869,7 +1890,7 @@ ProcessProtocolSetup (
     {
 	/*
 	 * If we sent a WantToClose message, but just got a ProtocolSetup,
-	 * we must cancel our WantToClose.  It is the responsiblity of the
+	 * we must cancel our WantToClose.  It is the responsibility of the
 	 * other client to send a WantToClose later on.
 	 */
 
@@ -1879,7 +1900,7 @@ ProcessProtocolSetup (
     IceReadCompleteMessage (iceConn, SIZEOF (iceProtocolSetupMsg),
 	iceProtocolSetupMsg, message, pStart);
 
-    if (!IceValidIO (iceConn))
+    if (!IceValidIO (iceConn) || pStart == NULL)
     {
 	IceDisposeCompleteMessage (iceConn, pStart);
 	return (0);
@@ -1930,6 +1951,13 @@ ProcessProtocolSetup (
     }
 
     EXTRACT_STRING (pData, swap, protocolName);
+    if (protocolName == NULL)
+    {
+	_IceErrorSetupFailed (iceConn, message->protocolOpcode,
+	    "protocolName allocation failed");
+	IceDisposeCompleteMessage (iceConn, pStart);
+	return (0);
+    }
 
     if (iceConn->process_msg_info)
     {
@@ -2555,4 +2583,3 @@ _IceProcessCoreMessage (
 const int		_IceVersionCount = 1;
 const _IceVersion	_IceVersions[] = {
 			    {IceProtoMajor, IceProtoMinor, _IceProcessCoreMessage}};
-
